package org.eco.als.service.impl;

import java.util.Arrays;
import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.eco.common.core.core.domain.model.LoginUser;
import org.eco.common.excel.entity.ExcelResultRes;
import org.eco.common.excel.service.IExcelService;
import org.eco.common.core.utils.bean.BeanUtils;
import org.eco.common.core.utils.MapstructUtils;
import org.eco.system.service.IImportExportService;
import org.eco.common.core.utils.StringUtils;
import org.eco.system.domain.bo.ImportExportBo;
import org.eco.common.orm.core.page.PageQuery;
import org.eco.common.core.core.page.PageResult;
import org.eco.common.orm.core.service.impl.BaseServiceImpl;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import java.io.IOException;
import org.eco.als.mapper.JudgeFaultLogicMapper;
import org.eco.als.domain.JudgeFaultLogic;
import org.eco.als.domain.bo.JudgeFaultLogicBo;
import org.eco.als.domain.vo.JudgeFaultLogicVo;
import org.eco.als.domain.vo.JudgeFaultLogicImportVo;
import org.eco.als.listener.JudgeFaultLogicImportListener;
import org.eco.als.service.IJudgeFaultLogicService;
import static org.eco.als.domain.table.JudgeFaultLogicTableDef.JUDGE_FAULT_LOGIC;

/**
 * 判故逻辑Service业务层处理
 *
 * @author wgk
 * @date 2024-12-11
 */
@Service
@Slf4j
public class JudgeFaultLogicServiceImpl extends BaseServiceImpl<JudgeFaultLogicMapper, JudgeFaultLogic> implements IJudgeFaultLogicService {
    @Resource
    private JudgeFaultLogicMapper judgeFaultLogicMapper;

    @Resource
    private IExcelService excelService;

    @Resource
    private IImportExportService importExportService;

    @Override
    public QueryWrapper query() {
        return super.query().from(JUDGE_FAULT_LOGIC);
    }

    private QueryWrapper buildQueryWrapper(JudgeFaultLogicBo judgeFaultLogicBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.or(JUDGE_FAULT_LOGIC.MODEL.like
        (judgeFaultLogicBo.getKeyWord()));
        queryWrapper.or(JUDGE_FAULT_LOGIC.HMC_CODE.like
        (judgeFaultLogicBo.getKeyWord()));

        return queryWrapper;
    }

    /**
     * 查询判故逻辑
     *
     * @param id 判故逻辑主键
     * @return 判故逻辑
     */
    @Override
    public JudgeFaultLogicVo selectById(Long id) {
            return this.getOneAs(query().where(JUDGE_FAULT_LOGIC.ID.eq(id)), JudgeFaultLogicVo.class);

    }


    /**
     * 查询判故逻辑列表
     *
     * @param judgeFaultLogicBo 判故逻辑Bo
     * @return 判故逻辑集合
     */
    @Override
    public List<JudgeFaultLogicVo> selectList(JudgeFaultLogicBo judgeFaultLogicBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(judgeFaultLogicBo);
            return this.listAs(queryWrapper, JudgeFaultLogicVo.class);
    }

    /**
     * 分页查询判故逻辑列表
     *
     * @param judgeFaultLogicBo 判故逻辑Bo
     * @return 分页判故逻辑集合
     */
    @Override
    public PageResult<JudgeFaultLogicVo> selectPage(JudgeFaultLogicBo judgeFaultLogicBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(judgeFaultLogicBo);
            Page<JudgeFaultLogicVo> page = this.pageAs(PageQuery.build(), queryWrapper, JudgeFaultLogicVo.class);
        return PageResult.build(page);
    }

    /**
     * 新增判故逻辑
     *
     * @param judgeFaultLogicBo 判故逻辑Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(JudgeFaultLogicBo judgeFaultLogicBo) {
    JudgeFaultLogic judgeFaultLogic =MapstructUtils.convert(judgeFaultLogicBo, JudgeFaultLogic. class);

        return this.save(judgeFaultLogic);//使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 新增判故逻辑，前台提供主键值，一般用于导入的场合
     *
     * @param judgeFaultLogicBo 判故逻辑Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insertWithPk(JudgeFaultLogicBo judgeFaultLogicBo)
    {
        JudgeFaultLogic judgeFaultLogic = MapstructUtils.convert(judgeFaultLogicBo, JudgeFaultLogic.class);


            return judgeFaultLogicMapper.insertWithPk(judgeFaultLogic) > 0;//前台传来主键值
    }

    /**
     * 修改判故逻辑
     *
     * @param judgeFaultLogicBo 判故逻辑Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(JudgeFaultLogicBo judgeFaultLogicBo) {
        JudgeFaultLogic judgeFaultLogic =MapstructUtils.convert(judgeFaultLogicBo, JudgeFaultLogic. class);
        if (ObjectUtil.isNotNull(judgeFaultLogic) && ObjectUtil.isNotNull(judgeFaultLogic.getId())){
            boolean updated = this.updateById(judgeFaultLogic);
                return updated;
        }
        return false;
    }

    @Override
    public void asyncImportData(MultipartFile file, boolean updateSupport, LoginUser loginUser) {
        ExcelResultRes result;
        try {
            String name = file.getOriginalFilename();
            result = excelService.importExcel(file.getInputStream(), name, JudgeFaultLogicImportVo.class, new JudgeFaultLogicImportListener(updateSupport, loginUser));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        boolean flag = importExportService.saveInfo(result, loginUser, "0");
        if (flag) {
            log.info("异步导入日志写入成功");
        }
    }

    @Override
    public void asyncExport(List<JudgeFaultLogicVo> listVo, String sheetName, LoginUser loginUser) {
        ExcelResultRes result = excelService.exportExcel(listVo, sheetName, JudgeFaultLogicVo.class);
        boolean flag = importExportService.saveInfo(result, loginUser, "1");
        if (flag) {
            log.info("异步导出日志写入成功");
        }
    }

    /**
     * 批量删除判故逻辑
     *
     * @param ids 需要删除的判故逻辑主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

}
